package plugins.big.bigsnakeutils.process.process2D;

import plugins.big.bigsnakeutils.process.process1D.Convolver1D;

public class LoG2D {
	/**
	 * Apply a Laplacian of Gaussian 2D. Separable implementation.
	 */
	public static double[] doLoG_Separable(double[] input, int nx, int ny,
			double sigmaX, double sigmaY) {
		if (input == null)
			return null;
		int d = 0; // dimension
		d = (sigmaX > 0 ? d + 1 : d);
		d = (sigmaY > 0 ? d + 1 : d);

		if (d == 0)
			return input;

		double pd = Math.pow(2 * Math.PI, d / 2.0);
		double sx = (sigmaX > 0 ? sigmaX : 1.0);
		double sy = (sigmaY > 0 ? sigmaY : 1.0);
		double cst = 1.0 / (pd * sx * sy);

		double kernelFactX[] = createKernelLoG_Fact(sigmaX, cst);
		double kernelBaseX[] = createKernelLoG_Base(sigmaX);
		double kernelFactY[] = createKernelLoG_Fact(sigmaY, cst);
		double kernelBaseY[] = createKernelLoG_Base(sigmaY);

		double[] outputX = new double[nx * ny];
		double[] outputY = new double[nx * ny];

		if (sigmaY > 0.0) {
			double vinY[] = new double[ny];
			double voutY[] = new double[ny];
			for (int x = 0; x < nx; x++) {
				for (int y = 0; y < ny; y++)
					vinY[y] = outputX[(nx * y) + x];
				Convolver1D.convolve(vinY, voutY, kernelFactY);
				for (int y = 0; y < ny; y++)
					outputX[(nx * y) + x] = voutY[y];

				for (int y = 0; y < ny; y++)
					vinY[y] = outputY[(nx * y) + x];
				Convolver1D.convolve(vinY, voutY, kernelBaseY);
				for (int y = 0; y < ny; y++)
					outputY[(nx * y) + x] = voutY[y];
			}
		}
		if (sigmaX > 0.0) {
			double vinX[] = new double[nx];
			double voutX[] = new double[nx];
			for (int y = 0; y < ny; y++) {
				for (int x = 0; x < nx; x++)
					vinX[x] = outputX[(nx * y) + x];
				Convolver1D.convolve(vinX, voutX, kernelBaseX);
				for (int x = 0; x < nx; x++)
					outputX[(nx * y) + x] = voutX[x];

				for (int x = 0; x < nx; x++)
					vinX[x] = outputY[(nx * y) + x];
				Convolver1D.convolve(vinX, voutX, kernelFactX);
				for (int x = 0; x < nx; x++)
					outputY[(nx * y) + x] = voutX[x];
			}
		}

		for (int x = 0; x < nx; x++)
			for (int y = 0; y < ny; y++)
				outputX[(nx * y) + x] += outputY[(nx * y) + x];

		return outputX;
	}

	/**
	 * cst*(x^2/(sigma^4)-1/(sigma^2))*exp(-(x^2)/(sigma^2))
	 */
	public static double[] createKernelLoG_Fact(double sigma, double cst) {

		if (sigma <= 0.0) {
			double[] kernel = new double[1];
			kernel[0] = 1.0;
			return kernel;
		}

		double s2 = sigma * sigma;
		double s4 = s2 * s2;
		double dem = 2.0 * s2;
		int size = (int) Math.round(((int) (sigma * 3.0)) * 2.0 + 1); // always
																		// odd
																		// size
		int size2 = size / 2;
		double[] kernel = new double[size];

		double x;
		for (int k = 0; k < size; k++) {
			x = (k - size2) * (k - size2);
			kernel[k] = cst * (x / s4 - 1.0 / s2) * Math.exp(-x / dem);
		}
		return kernel;
	}

	/**
	 * exp(-(x^2)/(sigma^2))
	 */
	public static double[] createKernelLoG_Base(double sigma) {

		if (sigma <= 0.0) {
			double[] kernel = new double[1];
			kernel[0] = 1.0;
			return kernel;
		}

		double s2 = sigma * sigma;
		double dem = 2.0 * s2;
		int size = (int) Math.round(((int) (sigma * 3.0)) * 2.0 + 1); // always
																		// odd
																		// size
		int size2 = size / 2;
		double[] kernel = new double[size];

		double x;
		for (int k = 0; k < size; k++) {
			x = (k - size2) * (k - size2);
			kernel[k] = Math.exp(-x / dem);
		}
		return kernel;
	}

}
